Порядок действий:
1. С помощью CMAKE откомпилировать библиотеку решателей paralution, находящуюся в папке paralution-1.1.0.
В настройках CMAKE можно выбрать, с какой технологией распараллеливания будут работать решатели: OpenMP, OpenCL или CUDA.
При этом OpenMP работает только на CPU, OpenCL работает как на CPU, так и на GPU, а CUDA работает только на GPU от Nvidia.
По умолчанию всегда используется технология OpenMP.
2. С помощью проекта CodeBlocks (файл math.cbp) или make-файла откомпилировать библиотеку math, находящуюся в папке math.
По умолчанию включена поддержка как решателей paralution (-DNLA3D_USE_PARALUTION), так и решателей mkl (-DNLA3D_USE_MKL). При необходимости, ненужное отключить в соответствующих настройках проекта CodeBlocks или в make-файле.
Особое внимание уделить файлу EquationSolver.h и значениям абсолютной невязки "double abs_tol = 1.0E-06" и относительной невязки "double rel_tol = 1.0E-06" для решателей paralution. Это критерии выхода для итерационного решателя. В зависимости от требуемой точности решения их нужно изменять.
3. С помощью проекта CodeBlocks (файл nla3d.cbp) или make-файла откомпилировать библиотеку nla3d, находящуюся в папке nla3d.
В данной версии nla3d улучшен алгоритм вывода данных для VTK-файлов. Скорость вывода увеличена в разы, особенно хорошо это заметно на больших сетках.
Для успешной компиляции необходимо наличие исходников библиотеки линейной алгебры eigen.
В ходе компиляции будет большое количество warning'ов, это нормально и связано с eigen.
4. С помощью CMAKE откомпилировать библиотеку comsdk (https://sa2systems.ru:881/com/comsdk).
5. Создать проект библиотеки gcdfes, состоящий из четырёх файлов - gcdfes.cpp (https://sa2systems.ru:881/gcd/gcdfes/blob/master/dev/gcdfes.cpp), gcdfes.h (https://sa2systems.ru:881/gcd/gcdfes/blob/master/dev/gcdfes.h) и FEReaders.cpp/FEReaders.h (внутри папки libgcdfes).
При создании библиотеки указать, какие решатели будем использовать - paralution (-DNLA3D_USE_PARALUTION), mkl (-DNLA3D_USE_MKL) или оба сразу.
Пример использования решателей mkl внутри кода: math::PARDISO_equationSolver eqSolver = math::PARDISO_equationSolver();
Пример использования решателей paralution внутри кода: math::PARALUTION_equationSolver eqSolver = math::PARALUTION_equationSolver();
При создании библиотеки gcdfes для линковщика подключить следующие библиотеки в следующем порядке (для варианта, когда используются как решатели mkl, так и решатели paralution): -lcomsdk -lnla3d -lmath -lparalution -lmkl_intel_lp64 -lmkl_sequential -lmkl_core -lpthread -lm -ldl -lboost_system
Пример проекта CodeBlocks (libgcdfes.cbp) и make-файла находятся в папке libgcdfes (для корректной работы нужно добавить два файла - gcdfes.cpp и gcdfes.h).
6. Создать проект решаемой задачи (проект исполняемого файла). Допустим, он будет называться gcdfes (пример проекта CodeBlocks - gcdfes.cbp и make-файла есть в папке gcdfes). Тестовую задачу можно взять здесь: https://sa2systems.ru:881/gcd/gcdfes/blob/master/tests/test_gcdfes
Компилятору указать пути к корневой директории проектов, директориям comsdk, easylogging++, libgcdfes, math, nla3d и eigen.
Для линковщика подключить следующие библиотеки в следующем порядке: -lcomsdk -lnla3d -lmath -lboost_system
В папку с полученным исполняемым файлом поместить библиотеку libgcdfes.so, полученную в предыдущем пункте, а также необходимые файлы задачи - *.tsk, *.tsl, *.bc, *.dot, *.neu, ...
Для запуска программы нужно в LD_LIBRARY_PATH прописать пути к библиотекам mkl (если используются решатели mkl) и/или paralution (если используются решатели paralution).
В качестве альтернативы можно использовать sh-скрипт запуска. Пример sh-скрипта есть в папке gcdfes.

~~~
ВОЗМОЖНЫЕ ПРОБЛЕМЫ И ПУТИ ИХ РЕШЕНИЯ:
* При использовании старых версий компилятора может потребоваться включить поддержку стандарта C++11
(в новых версиях включено по умолчанию). Делается это с помощью опции "-std=c++11".
* Компилятор может ругаться на функцию "sprintf_s". Её можно заменить на "snprintf", которая в данном случае полностью аналогична.
* В библиотеке comsdk присутствует не самая удачная реализация функции запуска внешних программ (файл external_binary_executor.cpp).
Реализация выполнена с помощью экспериментальных функций boost, которые не входят в релизный набор функций boost
и для некоторых версий boost эти функции могут вызывать конфликт.
В этом случае можно удалить (закомментировать) куски кода в файле external_binary_executor.cpp между
"#if defined __linux__" и "#endif". После этого запуск внешних программ станет невозможным, но пока он и не нужен.
